from flask import Flask
from flask import render_template
from flask import redirect
from flask import url_for
from flask import make_response
from flask import request

import json

from datetime import datetime, timedelta
from threading import Thread
from time import sleep

from w1thermsensor import W1ThermSensor
import RPi.GPIO as GPIO

sensor = W1ThermSensor()
water_pin = 26  #17/27/22
heat_pin = 20

heat_state = False

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
GPIO.setup(water_pin, GPIO.OUT)

class Heat(Thread):
    def __init__(self, pin):
        super(Heat, self).__init__()
        self.__pin = pin
        self.__state = False
        GPIO.setup(pin, GPIO.OUT)
        if GPIO.input(pin):
            self.__state = True
        self.__run_condition = True
        self.__next_update = datetime.now()

    def on(self):
        self.__state = True
        GPIO.output(self.__pin, 1)

    def off(self):
        self.__state = False
        GPIO.output(self.__pin, 0)

    def run(self):
        self.__next_update = datetime.now()
        while self.__run_condition:
            now = datetime.now()
            if now >= self.__next_update:
                if self.__state:
                    # Do a power cycle to prevent auto-poweroff
                    GPIO.output(self.__pin, 0)
                    sleep(5)
                    GPIO.output(self.__pin, 1)
                self.__next_update = now + timedelta(minutes=5)
            sleep(1)

    def stop(self):
        self.__run_condition = False
        self.join()


    def state(self):
        return self.__state


heat = Heat(heat_pin)
heat.start()

app = Flask(__name__)


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/sample', methods=['GET'])
def get_sample():
    global heat
    global sensor
    try:
        temperature = f"{float(sensor.get_temperature()):.1f}"
    except SensorNotReadyError:
        temperature = None

    water_state = False
    if GPIO.input(water_pin):
        water_state = True

    res = {}
    res["id"] = str(1)
    if(temperature):
        res["temperature"] = temperature
    res["water"] = water_state
    res["heat"] = heat.state()
    return res


@app.route('/sample', methods=['PATCH'])
def patch_reroute():
    global heat
    record = json.loads(request.data)
    if "water" in record:
        water_state = record["water"]
        if water_state:
            GPIO.output(water_pin, 1)
        else:
            GPIO.output(water_pin, 0)
    if "heat" in record:
        heat_state = record["heat"]
        if heat_state:
            heat.on()
        else:
            heat.off()

    res = make_response("", 204)
    return res